package com.nsgk.agentcentersdk.api;

import cn.hutool.core.util.URLUtil;
import com.nsgk.agentcentersdk.core.NSReportObject;
import com.nsgk.agentcentersdk.entity.NSEntityBase;
import com.nsgk.agentcentersdk.network.NSNetworkManager;
import com.nsgk.agentcentersdk.network.NSNetworkRequest;
import com.nsgk.agentcentersdk.network.NSNetworkResponse;

// SDK客户端
/*
请求:
    <host>:<port>/<api>?identifier=<identifier>&protocol=<protocol>&timestamp=<timestamp>
        header: sign
        body: 加密json
响应:
    body:
        code: 200
        msg: ""
        timestamp:
        data: "" // 加密json
 */
public final class NSSDKClient
{
    private final String host;
    private final short port;
    private final String identifier;
    private final String publicKey;

    NSSDKClient(String host, short port, String identifier, String publicKey)
    {
        this.host = host;
        this.port = port;
        this.identifier = identifier;
        this.publicKey = publicKey;
    }

    // 发送请求
    public <T extends NSEntityBase> NSApiResult Send(int protocol, T object)
    {
        String url;
        NSReportObject<T> reportObject;
        NSNetworkRequest request;

        url = NSApi.NS_API_REPORT;
        reportObject = new NSReportObject<>(identifier, protocol, object);
        request = new NSNetworkRequest();
        request.setUrl(BuildUrl(url));
        request.WriteDataObject(reportObject, publicKey);
        NSNetworkResponse response = NSNetworkManager.Post(request);
        String json = response.getData();
        return NSApiResult.FromJSON(json);
    }

    private String BuildUrl(String url)
    {
        return URLUtil.completeUrl(host + ":" + port, url);
    }
}
