package com.nsgk.agentcentersdk.core;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson2.JSON;
import com.nsgk.agentcentersdk.utility.NSSignTool;
import com.nsgk.agentcentersdk.utility.NSStr;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Map;

// 封装对象
@Data
@Accessors(chain = true)
public class NSReportObject<T>
{
    protected static final String[] SIGN_FIELDS = {
            "identifier", "protocol", "timestamp", "dataLength",
    };
    private String identifier; // 识别ID
    private String sign; // 签名
    private int protocol = NSProtocol.NS_PROTOCOL_INVALID; // 协议
    private Long timestamp; // 客户端时间戳
    private T data;
    private String dataStr = "";

    public NSReportObject<T> setData(T d)
    {
        data = d;
        dataStr = "";
        if(null != data)
            dataStr = JSON.toJSONString(data);
        return this;
    }

    public NSReportObject<T> SetDataStr(String str, Class<T> clazz)
    {
        data = JSON.parseObject(str, clazz);
        dataStr = str;
        return this;
    }

    public NSReportObject()
    {
    }

    public NSReportObject(String identifier, int protocol)
    {
        this(identifier, protocol, null);
    }

    public NSReportObject(String identifier, int protocol, T data)
    {
        this.identifier = identifier;
        this.protocol = protocol;
        timestamp = System.currentTimeMillis();
        setData(data);
        sign = Sign();
    }

    public boolean IsValid()
    {
        return NSStr.IsNotEmpty(identifier) && null != timestamp && protocol > 0;
    }

    public String Sign()
    {
        Map<String, Object> map;

        map = BeanUtil.beanToMap(this);
        map.put("dataLength", NSStr.Length(dataStr));
        return NSSignTool.Sign(map, SIGN_FIELDS);
    }

    public boolean CheckSign(String src)
    {
        if(!IsValid())
            return false;
        return src.equals(Sign());
    }

    public NSReportObject<T> SetTimestampStr(String str)
    {
        timestamp = Long.parseLong(str);
        return this;
    }

    public NSReportObject<T> SetProtocolStr(String str)
    {
        protocol = NSProtocol.FromString(str);
        return this;
    }
}
